#pragma rtGlobals=1		// Use modern global access method.
#pragma rtGlobals=1		// Use modern global access method.

////////////////////////////////////////////////////////////////////////////////////////////////////

//Procedures for controlling a Lumencor SOLA light engine (http://lumencor.com/product/sola-light-engine/)
//08/10/2012 Josef G. Trapani (jtrapani@amherst.edu)
//Note: Procedure requires the VDT2.xop extension to be installed
//Note: User must install the FTDI USB Serial driver for the USB to Serial (RS232) cable for the Lumencor: http://www.ftdichip.com/Drivers/VCP.htm


////////////////////////////////////////////////////////////////////////////////////////////////////

Menu "Macros"

	"Lumencor SOLA",Start_SOLA_Panel("start")
	
End

////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////
// Panel for SOLA 

Function Start_SOLA_Panel(ctrlName)
	String ctrlName	 
	String savedDataFolder = GetDataFolder(1)	// Save

	DoWindow SOLA_Panel
	If (V_flag!=0)
		KillWindow SOLA_Panel
		SetDataFolder root:SOLA
		KillWaves /Z /A
		KillDataFolder /Z  root:SOLA
	Endif
	
	If (cmpstr(ctrlName,"start")==0)
		NewDataFolder /O/S root:SOLA
		Variable /G SOLA_intensity = 40		// DAC intensity 0xFF is full off and 0x00 is full on
		Variable /G SOLA_On = 0		// 0 is Off and 1 is On
		Variable /G SOLA_Temp			// SOLA Temp sensor
		String /G SOLA_serialName		// SOLA serial device name
	Endif

	SetDataFolder savedDataFolder			// and restore
	
	Variable returnNum = initSOLA(ctrlName)

	If (returnNum==-1)
		Abort "No Serial Devices Found! ---> Check menu /MISC/VDT2/"
	Elseif (returnNum==0)
		Print "Lumencor SOLA panel closed. Light is off."
	Elseif (returnNum==1)
		SOLA_Panel(ctrlName)
	Endif

End

////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////
//Initialize SOLA Serial Devices

Function initSOLA(ctrlName)
	String ctrlName
	String cmd
	String tempSerialList
	String tempSerialName
	String platform= UpperStr(igorinfo(2))


	SVAR /z SerialName = root:SOLA:SOLA_serialName

	If ((cmpstr(ctrlName,"start")==0)&&(SVAR_Exists(SerialName)==1))

		VDTGetPortList2
		tempSerialList = S_VDT

		//************
		//************
		//Note: User should check the VDT2 menu for the name of their USB Serial device and adjust the following code accordingly:

		If (cmpstr(platform,"WINDOWS")==0)
			//Determine Serial Device Names
			tempSerialName = StringFromList(1, tempSerialList,";")
			If (cmpstr(tempSerialName, "COM4")!=0)
				KillDataFolder /Z  root:SOLA
				return -1
			Else
				SerialName = StringFromList(1, tempSerialList,";")
			Endif
		Elseif (cmpstr(platform,"MACINTOSH")==0)
			//Determine Serial Device Names
			tempSerialName = StringFromList(0, tempSerialList,";")
			If (cmpstr(tempSerialName[0], "u")!=0)
				KillDataFolder /Z  root:SOLA
				return -1
			Else
				SerialName = StringFromList(0, tempSerialList,";")
			Endif
		Else
			KillDataFolder /Z  root:SOLA
			return -1	
		Endif

		VDTOperationsPort2 $(SerialName)
		VDT2 baud=9600, stopbits=1, databits=8, parity=0

		//Set GPIO 0-3 as open drain output "57 02 FF 50"
		VDTWriteBinary2 /TYPE=(0x48) 0x57, 0x02, 0xFF, 0x50		
		
		//Set GPIO 5-7 push-pull out, GPI0 4 open drain out "57 03 AB 50"
		VDTWriteBinary2 /TYPE=(0x48) 0x57, 0x03, 0xAB, 0x50		

		VDTWriteBinary2 /TYPE=(0x48) 0x4F, 0xFF, 0x50  //Full Off "4FFF50"
		return 1

	ElseIf (cmpstr(ctrlName,"closePanelbutton")==0)
		VDTWriteBinary2 /TYPE=(0x48) 0x4F, 0xFF, 0x50  //Full Off "4FFF50"
		return 0
	Endif

End


////////////////////////////////////////////////////////////////////////////////////////
// SOLA Controller Panel


Static Function SOLA_Panel(ctrlName)
	String ctrlName
	Variable Xcoord=200
	Variable Ycoord=60

	If (cmpstr(ctrlName,"start")==0)
	
		String info = IgorInfo(0)
		String screen1RectStr = StringByKey("SCREEN1", info)		//e.g., "DEPTH=23,RECT=0,0,1280,1024"
		Variable depth, left, top, right, bottom
		sscanf screen1RectStr, "DEPTH=%d,RECT=%d,%d,%d,%d", depth, left,top, right, bottom
	
		top+=40 //adjust for title bar
		String platform= UpperStr(igorinfo(2))
		If (cmpstr(platform,"WINDOWS")==0)
			NewPanel /N=SOLA_Panel /W=(right-300,top,right,300) /FLT=2 /K=2 as "SOLA Driver v1.1"
		Else	
			NewPanel /N=SOLA_Panel /W=(right-380,top,right-80,300) /FLT=2 /K=2 as "SOLA Driver v1.1"
		Endif
		SetActiveSubwindow _endfloat_
	
		ModifyPanel /W=SOLA_Panel cbRGB=(62535,62535,62535), frameStyle=3, frameInset=5, noEdit=1, fixedSize=1
		SetDrawLayer /W=SOLA_Panel UserBack
		SetDrawEnv /W=SOLA_Panel fillfgc= (45535,46535,46214)
		DrawRect /W=SOLA_Panel 10,10,290,50
		SetDrawEnv /W=SOLA_Panel linefgc= (21845,21845,21845)
		DrawText /W=SOLA_Panel 20,40,"Lumencor SOLA Light Driver"
	
		Button closePanelbutton,pos={210,20},size={60,20},proc=Start_SOLA_Panel,title="Close", win=SOLA_Panel

	
		CheckBox check1 pos={Xcoord-180,Ycoord+40}, title="white",mode=0, value=1,disable=0,proc=SOLAcheck, win=SOLA_Panel
		CheckBox check2 pos={Xcoord-180,Ycoord+60}, title="blue",mode=0, value=0,disable=0,proc=SOLAcheck, win=SOLA_Panel
		CheckBox check3 pos={Xcoord-180,Ycoord+80}, title="red",mode=0, value=0,disable=0,proc=SOLAcheck, win=SOLA_Panel
		Checkbox check4,pos={Xcoord-180,Ycoord},size={40,10},value=0,disable=0,proc=BigSOLAPanel,title="Big \rButton", win=SOLA_Panel
	
		Button button3,pos={Xcoord-100,Ycoord+20},size={100,40},disable=0,proc=SOLA_Start,title="Light is Off", win=SOLA_Panel
		Button button3,fSize=16,fStyle=16,fColor=(255,255,255), win=SOLA_Panel
	
		
		NVAR gIntensity= root:SOLA:SOLA_intensity
		Slider setvar1,pos={Xcoord+20,Ycoord},size={100,100},proc=updateIntensity, title="% Intensity", win=SOLA_Panel
		Slider setvar1, disable=2, live=0, fSize=12,limits={0,100,20}, value=gIntensity, disable=0, win=SOLA_Panel


		NVAR gTemp = root:SOLA:SOLA_Temp		
		SetVariable setvar2,pos={Xcoord-180,Ycoord+150},size={160,1},limits={1,1000,0},title="Sensor Temp:",fSize=14, proc=readSOLAtemp, win=SOLA_Panel
		SetVariable setvar2, disable=2, value=gTemp, win=SOLA_Panel

	Endif

End Function

////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////
//SOLA checkbox for color selection

Function SOLAcheck (ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked

	NVAR /z event = root:SOLA:SOLA_On

	If (checked==1)			// 1 if selelcted, 0 if not

		If (cmpstr(ctrlName,"check1")==0) //white checkbox
			CheckBox check1, disable=2, value=01, win=SOLA_Panel
			CheckBox check2, disable=0, value=0, win=SOLA_Panel
			CheckBox check3, disable=0, value=0, win=SOLA_Panel
			Slider setvar1 disable=0, win=SOLA_Panel	
	
		Elseif (cmpstr(ctrlName,"check2")==0) //blue checkbox
			CheckBox check1, disable=0, value=0, win=SOLA_Panel
			CheckBox check2, disable=2, value=01, win=SOLA_Panel
			CheckBox check3, disable=0, value=0, win=SOLA_Panel
			Slider setvar1 disable=1, win=SOLA_Panel	
	
		Elseif (cmpstr(ctrlName,"check3")==0) //red checkbox
			CheckBox check1, disable=0, value=0, win=SOLA_Panel
			CheckBox check2, disable=0, value=0, win=SOLA_Panel
			CheckBox check3, disable=2, value=1, win=SOLA_Panel

		Endif
	
		If (event==1) //If light is ON
			SOLA_start (ctrlName)
		Endif

	Endif
	
End Function

////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////
//Big Button popup window
Function BigSOLAPanel (ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	If (checked==1)			// 1 if selelcted, 0 if not
	
		String info = IgorInfo(0)
		String screen1RectStr = StringByKey("SCREEN1", info)		//e.g., "DEPTH=23,RECT=0,0,1280,1024"
		Variable depth, left, top, right, bottom
		sscanf screen1RectStr, "DEPTH=%d,RECT=%d,%d,%d,%d", depth, left,top, right, bottom
	
		String platform= UpperStr(igorinfo(2))
		If (cmpstr(platform,"WINDOWS")==0)
			NewPanel /N=BigSOLA_Panel /W=(right-250,top,right,150) /FLT=2
		Else	
			NewPanel /N=BigSOLA_Panel /W=(right-300,top,right-50,150) /FLT=2
		Endif
	
		ModifyPanel cbRGB=(30583,30583,30583)
		DoWindow /HIDE=1 /W=SOLA_Panel SOLA_Panel
		SetDrawLayer UserBack
		DrawRRect 16,15,240,135
		Button button0,pos={30,25},size={200,100},proc=SOLA_Start,title="On/Off",fSize=58
		Checkbox check4,pos={0,0},size={40,10},value=1,disable=0,proc=BigSOLAPanel,title="Close"
	Else
		DoWindow /F /HIDE=0 /W=SOLA_Panel SOLA_Panel
		Checkbox check4,value=0, win=SOLA_Panel
		Killwindow BigSOLA_Panel
	Endif
End Function

////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////
//Light On/Off SOLA

Function SOLA_start (ctrlName) : ButtonControl
	String ctrlName //SOLA color selection
	
	NVAR /z event = root:SOLA:SOLA_On
	NVAR /z gIntensity = root:SOLA:SOLA_intensity


	If (event==0)

		controlInfo /W=SOLA_Panel check1  //Determine Color choice
		variable checknum1 = V_Value		
		controlInfo /W=SOLA_Panel check2  //Determine Color choice
		variable checknum2 = V_Value		
		controlInfo /W=SOLA_Panel check3  //Determine Color choice
		variable checknum3 = V_Value		
	
		If  (checknum1==1) //white

			VDTWriteBinary2 /TYPE=(0x48) 0x4F, 0x00, 0x50 //Full On "4F 00 50"
			Slider setvar1 disable=0, win=SOLA_Panel
			Button button3, win=SOLA_Panel, title="Light is On", fColor=(65535,65535,65535)
			Button /Z button0, win=BigSOLAPanel, title="Light is On", fColor=(65535,20000,20000)
			updateIntensity (ctrlName, gIntensity, event)
	
		ElseIf  (checknum2==1)  //blue

			VDTWriteBinary2 /TYPE=(0x48) 0x4F, 0x5F, 0x50 //Enables Blue
			Button button3, win=SOLA_Panel, title="Light is On", fColor=(20000,20000,65535)
			Button /Z button0, win=BigSOLAPanel, title="Light is On", fColor=(65535,20000,20000)

		ElseIf  (checknum3==1)  //red

			VDTWriteBinary2 /TYPE=(0x48) 0x4F, 0x7E, 0x50 //Enables Red
			Button button3, win=SOLA_Panel, title="Light is On", fColor=(65535,20000,20000)
			Button /Z button0, win=BigSOLAPanel, title="Light is On", fColor=(65535,20000,20000)

		Endif	
		
		//Switch the on/off flag "event"
		event=1

	ElseIf  ((event==1)||(cmpstr(ctrlName,"SOLAcheck")==0))
		VDTWriteBinary2 /TYPE=(0x48) 0x4F, 0xFF, 0x50 //Full Off "4FFF50"
		Button button3, win=SOLA_Panel, title="Light is Off",fColor=(255,255,255)
		Button /Z button0, win=BigSOLAPanel, title="Light is Off",fColor=(255,255,255)
		//Switch the on/off flag "event"
		event=0 
	Endif

	readSOLAtemp(ctrlName)
	ControlUpdate /A /W=SOLA_Panel

End


////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////
//Update SOLA Intensity

Function updateIntensity (name, value, event) : SliderControl
	String name	// name of this slider control
	Variable value	// value of slider
	Variable event	// bit field: bit 0: value set; 1: mouse down, 2: mouse up, 3: mouse moved

	NVAR gIntensity = root:SOLA:SOLA_intensity		// SOLA Intensity

	gIntensity = value
	
	
	If (gIntensity==100)
		VDTWriteBinary2 /TYPE=(0x48) 0x53, 0x18, 0x03, 0x0F, 0xF0, 0x00, 0x50		// Full on Violet, Cyan, Green, Red
		VDTWriteBinary2 /TYPE=(0x48) 0x53, 0x1A, 0x03, 0x0F, 0xF0, 0x00, 0x50		// Full on Blue and Teal

	Elseif (gIntensity==80)
		VDTWriteBinary2 /TYPE=(0x48) 0x53, 0x18, 0x03, 0x0F, 0xF1, 0xF0, 0x50		
		VDTWriteBinary2 /TYPE=(0x48) 0x53, 0x1A, 0x03, 0x0F, 0xF1, 0xF0, 0x50		

	Elseif (gIntensity==60)
		VDTWriteBinary2 /TYPE=(0x48) 0x53, 0x18, 0x03, 0x0F, 0xF3, 0xF0, 0x50	
		VDTWriteBinary2 /TYPE=(0x48) 0x53, 0x1A, 0x03, 0x0F, 0xF3, 0xF0, 0x50	

	Elseif (gIntensity==40)
		VDTWriteBinary2 /TYPE=(0x48) 0x53, 0x18, 0x03, 0x0F, 0xF7, 0xF0, 0x50	
		VDTWriteBinary2 /TYPE=(0x48) 0x53, 0x1A, 0x03, 0x0F, 0xF7, 0xF0, 0x50

	Elseif (gIntensity==20)
		VDTWriteBinary2 /TYPE=(0x48) 0x53, 0x18, 0x03, 0x0F, 0xFB, 0xF0, 0x50
		VDTWriteBinary2 /TYPE=(0x48) 0x53, 0x1A, 0x03, 0x0F, 0xFB, 0xF0, 0x50

	Elseif (gIntensity==0)
		VDTWriteBinary2 /TYPE=(0x48) 0x53, 0x18, 0x03, 0x0F, 0xFF, 0xF0, 0x50		// Full off  Violet, Cyan, Green, Red
		VDTWriteBinary2 /TYPE=(0x48) 0x53, 0x1A, 0x03, 0x0F, 0xFF, 0xF0, 0x50		// Full off Blue and Teal
	Endif

End
	
////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////
//Read IIC Temp Sensor Strings

Function readSOLAtemp(ctrlName)
	String ctrlName
	Variable tempTemp

	NVAR gTemp = root:SOLA:SOLA_Temp		

	VDTWriteBinary2 /TYPE=(0x48) 0x53, 0x91, 0x02, 0x50		// Command String to the IIC Temp Sensor
	VDTReadBinary2 /TYPE=(0x50) tempTemp	// Read as unsigned 16-bit. Data is in most-significant 11 bits.
	tempTemp = trunc(tempTemp / 32)			// Shift right dropping 5 low-order bits.
	tempTemp *= 0.125							// Scale to degrees C.
	gTemp = tempTemp

End
////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////